import logging
import psycopg2

from sqlalchemy import event
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker, scoped_session
from sqlalchemy.orm.session import Session

from django.conf import settings

from gargantext.utils.json import json_dumps


__all__ = ['Session', 'session']


logger = logging.getLogger(__name__)


class TrackRollbackMixin(object):
    """Track whether or not session has been rollbacked"""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        event.listen(self, 'after_rollback', self._after_rollback, named=True)

    def close(self):
        if self.info.get('is_rollbacked'):
            self.info['is_rollbacked'] = False
        super().close()

    @property
    def is_rollbacked(self):
        return self.info.get('is_rollbacked')

    def _after_rollback(self, **kw):
        self.info['is_rollbacked'] = True


class ProtectedSession(TrackRollbackMixin, Session):
    """Imitate PostgREST authentication mecanism at each transaction"""

    def close(self):
        self._logout()
        super().close()

    logout = close

    def login(self, user):
        if settings.DEBUG and isinstance(user, str):
            # For debugging purposes
            from gargantext.models import User
            from sqlalchemy.orm.exc import NoResultFound
            username = user
            try:
                user = session.query(User).filter_by(username=username).one()
            except NoResultFound:
                raise Exception("User %s not found!" % username)

        self.user_id = user and user.id
        self.user_name = user and user.username
        self.role = settings.ROLE_SUPERUSER if user.is_superuser else \
                    settings.ROLE_STAFF if user.is_staff else \
                    settings.ROLE_USER if user.id else \
                    'anon'

        logger.debug("Plug authenticator for: %s (%s, %s)" % (
            self.role, self.user_name, self.user_id))

        event.listen(self, 'after_begin', self._postgres_auth)

    def _logout(self):
        if event.contains(self, 'after_begin', self._postgres_auth):
            logger.debug("Unplug authenticator for: %s (%s, %s)" % (
                self.role, self.user_name, self.user_id))
            event.remove(self, 'after_begin', self._postgres_auth)

        self.connection().info['authenticated'] = False
        self.user_id = self.user_name = self.role = None

    def _postgres_auth(self, *args, **kwargs):
        logger.debug("Authenticate in postgres as %s (%s, %s)" % (
            self.role, self.user_name, self.user_id))

        ops = [('set role %s', self.role),
               ('set local "request.jwt.claim.role" = \'%s\'', self.role),
               ('set local "request.jwt.claim.sub" = %r', self.user_name),
               ('set local "request.jwt.claim.user_id" = %d', self.user_id)]

        sql = ';'.join(s % p for s, p in ops if p) + ';'

        conn = self.connection()
        conn.info['authenticated'] = True
        conn.execute(sql)


# FIXME Keeped for backward compatibility but should be removed
engine = create_engine(settings.DATABASES['default']['SECRET_URL'])
session = scoped_session(sessionmaker(
    bind=engine.execution_options(isolation_level='READ COMMITTED'),
    autoflush=False))


protected_engine = create_engine(
    settings.DATABASES['protected']['SECRET_URL'],
    use_native_hstore=True,
    json_serializer=json_dumps,
    pool_size=20,
    max_overflow=0)


Session = sessionmaker(
    class_=ProtectedSession,
    # This is the default postgresql isolation level, we state it here
    # to be explicit
    bind=protected_engine.execution_options(isolation_level='READ COMMITTED'),
    # Disable autoflush to have more control over transactions
    autoflush=False,)


# FIXME Should rewrite bulk queries with SQLAlchemy Core: beware, those
#       functions are bypassing row level security!
# See: http://docs.sqlalchemy.org/en/latest/faq/performance.html#i-m-inserting-400-000-rows-with-the-orm-and-it-s-really-slow
def get_cursor():
    db_settings = settings.DATABASES['default']
    db = psycopg2.connect(**{
        'database': db_settings['NAME'],
        'user':     db_settings['USER'],
        'password': db_settings['PASSWORD'],
        'host':     db_settings['HOST'],
        'port':     db_settings['PORT']
    })
    return db, db.cursor()


class bulk_insert:
    def __init__(self, table, fields, data, cursor=None):
        # prepare the iterator
        self.iter = iter(data)
        # prepare the cursor
        if cursor is None:
            db, cursor = get_cursor()
            mustcommit = True
        else:
            mustcommit = False
        # insert data
        if not isinstance(table, str):
            table = table.__tablename__
        cursor.copy_from(self, table, columns=fields)
        # commit if necessary
        if mustcommit:
            db.commit()

    def read(self, size=None):
        # see http://www.postgresql.org/docs/9.4/static/sql-copy.html#AEN72054
        try:
            return '\t'.join(
                value.replace('\\', '\\\\').replace('\n', '\\\n').replace('\r', '\\\r').replace('\t', '\\\t')
                if isinstance(value, str) else str(value) if value is not None else '\\N'
                for value in next(self.iter)
            ) + '\n'
        except StopIteration:
            return ''

    readline = read


def bulk_insert_ifnotexists(model, uniquekey, fields, data, cursor=None, do_stats=False):
    """
    Inserts bulk data with an intermediate check on a uniquekey
    (ex: Ngram.terms) to see if the row existed before.

    If the row already existed we just retrieve its id.
    If it didn't exist we create it and retrieve the id.

    Returns a dict {uniquekey => id}

    Option:
        do stats: also returns the number of those that had no previous id
    """
    if cursor is None:
        db, cursor = get_cursor()
        mustcommit = True
    else:
        mustcommit = False
    # create temporary table with given data
    sql_columns = 'id INTEGER'

    cursor.execute('BEGIN WORK;')
    cursor.execute('LOCK TABLE %s IN SHARE ROW EXCLUSIVE MODE;' % model.__tablename__)


    for field in fields:
        column = getattr(model, field)
        sql_columns += ', %s %s' % (field, column.type, )
    cursor.execute('CREATE TEMPORARY TABLE __tmp__ (%s)' % (sql_columns, ))
    bulk_insert('__tmp__', fields, data, cursor=cursor)
    # update ids of the temporary table
    cursor.execute('''
        UPDATE __tmp__
        SET id = source.id
        FROM {sourcetable} AS source
        WHERE __tmp__.{uniquecolumn} = source.{uniquecolumn}
    '''.format(
        sourcetable = model.__tablename__,
        uniquecolumn = uniquekey,
    ))

    # insert what has not been found to the real table
    cursor.execute('''
        INSERT INTO {sourcetable} ({columns})
        SELECT {columns}
        FROM __tmp__
        WHERE id IS NULL
    '''.format(
        sourcetable = model.__tablename__,
        columns = ', '.join(fields),
    ))

    if do_stats:
        # remember how many rows we inserted just now
        n_new = cursor.rowcount

    # retrieve dict associating unique key to id
    cursor.execute('''
        SELECT source.id, source.{uniquecolumn}
        FROM {sourcetable} AS source
        INNER JOIN __tmp__ ON __tmp__.{uniquecolumn} = source.{uniquecolumn}
    '''.format(
        sourcetable = model.__tablename__,
        uniquecolumn = uniquekey,
        columns = ', '.join(fields),
    ))
    result = {
        # term : new_id
        row[1]: row[0] for row in cursor.fetchall()
    }
    # this is the end!
    cursor.execute('DROP TABLE __tmp__')
    if mustcommit:
        db.commit()

    if do_stats:
        return result, n_new
    else:
        return result

    cursor.execute('COMMIT WORK;')

    cursor.close()
