"""
Django settings for gargantext project.

Generated by 'django-admin startproject' using Django 2.0.3.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import os
import datetime
from gargantext.utils.config import config


# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/2.0/howto/deployment/checklist/


# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = config('SECRET_KEY')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = config('DEBUG', default=False, cast=bool)
MAINTENANCE = config('MAINTENANCE', default=False, cast=bool)

ALLOWED_HOSTS = config('ALLOWED_HOSTS', default='', cast=lambda v: list(filter(None, v.split(' '))))


# Application definition

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'rest_framework',
    'django_celery_beat',
    'gargantext.backend',
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'gargantext.backend.middleware.DatabaseSessionMiddleware',
]

ROOT_URLCONF = 'gargantext.backend.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = 'gargantext.backend.wsgi.application'


# Logging

LOG_LEVEL = config('LOG_LEVEL', default='DEBUG' if DEBUG else 'INFO')
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(name)s %(process)d %(threadName)s %(message)s'
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'file': {
            'level': LOG_LEVEL,
            'class': 'logging.FileHandler',
            'filename': config('LOG_FILE', default='gargantext.log'),
            'formatter': config('LOG_FORMATTER', default='verbose'),
        },
    },
    'loggers': {
        'django': {
            'handlers': ['file'],
            'level': LOG_LEVEL,
            'propagate': True,
        },
        # All django loggers: https://docs.djangoproject.com/fr/2.0/topics/logging/#id3
        'django.template': {
            # Don't keep debug logs for template module to avoid annoying and
            # useless noise, see:
            # https://github.com/encode/django-rest-framework/issues/3982#issuecomment-325290221
            'level': 'INFO' if LOG_LEVEL == 'DEBUG' else LOG_LEVEL,
        },
        'django.db.backends': {
            'level': 'INFO',
        },
        'gargantext': {
            'handlers': ['file'],
            'level': LOG_LEVEL,
            # Propagation to True means that this config applies to
            # 'gargantext' logger and all 'gargantext.*'
            'propagate': True,
        },
    },
}


# Database
# https://docs.djangoproject.com/en/2.0/ref/settings/#databases

DEFAULT_DATABASE = {
    'ENGINE': 'django.db.backends.postgresql_psycopg2',
    'NAME': config('DB_NAME', default='gargandb'),
    'USER': config('DB_USER', default='gargantua'),
    'PASSWORD': config('DB_PASS'),
    'HOST': config('DB_HOST', default='127.0.0.1'),
    'PORT': config('DB_PORT', default='5432'),
}
DATABASES = {
    'default': DEFAULT_DATABASE,
    'protected': dict(DEFAULT_DATABASE,
        USER=config('DB_PROTECTED_USER', default='authenticator'),
        PASSWORD=config('DB_PROTECTED_PASS'))
}
for db in DATABASES:
    DATABASES[db]['SECRET_URL'] = \
        'postgresql+psycopg2://{USER}:{PASSWORD}@{HOST}:{PORT}/{NAME}'.format(
            **DATABASES[db]
        )


# Password validation
# https://docs.djangoproject.com/en/2.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]


# Internationalization
# https://docs.djangoproject.com/en/2.0/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = config('TIME_ZONE', default='UTC')

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/2.0/howto/static-files/

STATIC_ROOT = os.path.join(BASE_DIR, 'static')
STATIC_URL = '/static/'

STATICFILES_DIRS = (
    os.path.join(BASE_DIR, 'gargantext/static'),
)

STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'django.contrib.staticfiles.finders.FileSystemFinder',
)

MEDIA_ROOT = os.path.join(BASE_DIR, 'media')
MEDIA_URL = '/media/'


# Asynchronous tasks

BROKER_URL = config('BROKER_URL', default='amqp://guest:guest@localhost:5672/')

CELERY_APP = config('CELERY_APP', default='gargantext.backend')
CELERY_ACCEPT_CONTENT = ['pickle', 'json', 'msgpack', 'yaml']
CELERY_TIMEZONE = TIME_ZONE
CELERYBEAT_SCHEDULER = 'django_celery_beat.schedulers:DatabaseScheduler'
CELERY_IMPORTS = ()


# REST-API

# See http://getblimp.github.io/django-rest-framework-jwt/#additional-settings
REST_FRAMEWORK = {
    'DEFAULT_PERMISSION_CLASSES': (
        'rest_framework.permissions.AllowAny',
    ),
    'DEFAULT_AUTHENTICATION_CLASSES': (
        'gargantext.backend.authentication.SessionAuthentication',
        'gargantext.backend.authentication.JWTAuthentication',
        'gargantext.backend.authentication.AnonymousAuthentication',
    ),
    'EXCEPTION_HANDLER': 'gargantext.backend.views.exception_handler',
}

# See http://getblimp.github.io/django-rest-framework-jwt/
JWT_AUTH = {
    'JWT_PAYLOAD_HANDLER': 'gargantext.backend.jwt.jwt_payload_handler',
    'JWT_PAYLOAD_GET_USER_ID_HANDLER':
        'gargantext.backend.jwt.jwt_get_user_id_from_payload_handler',
    'JWT_PAYLOAD_GET_USERNAME_HANDLER':
        'gargantext.backend.jwt.jwt_get_username_from_payload_handler',
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_SECRET_KEY': config('SECRET_KEY'),
    'JWT_EXPIRATION_DELTA': datetime.timedelta(seconds=36000),
    'JWT_AUTH_HEADER_PREFIX': 'Bearer',
    'JWT_AUTH_COOKIE': 'JWT' if DEBUG else None,
}

ROLE_SUPERUSER = 'gargantua'
ROLE_STAFF = 'gargandmin'
ROLE_USER = 'gargantext'


# Third-party web APIs

API_TOKENS = {
    "CERN": {
        "APIKEY":'b8514451-82d1-408e-a855-56d342a0b5f8',
        "APISECRET":'6680b13e-2b5a-4fba-8c0e-408884d5b904',
    },
    "MULTIVAC": {
        "APIKEY": "3a8ca010-1dff-11e7-97ef-a1a6aa4c2352"
    }
}


# BOOL Interpreter

BOOL_TOOLS_PATH = "gargantext/util/crawlers/sparql"
